#! /usr/bin/env python

import roslib; roslib.load_manifest('shell_execution_action_server')
import rospy

import actionlib

import shell_execution_action_server.msg
import os
import subprocess
import StringIO

class ShellCommand(object):
  # create messages that are used to publish feedback/result
  _feedback = shell_execution_action_server.msg.ShellCommandFeedback()
  _result   = shell_execution_action_server.msg.ShellCommandResult()

  def __init__(self, name):
    self._action_name = name
    self._as = actionlib.SimpleActionServer(self._action_name, shell_execution_action_server.msg.ShellCommandAction, execute_cb=self.execute_cb)
    self._as.start()
    self._feedback = shell_execution_action_server.msg.ShellCommandFeedback()
    
  def execute_cb(self, goal):
    
    # publish info to the console for the user
    rospy.loginfo('%s: Executing: %s' % (self._action_name, goal.command)) 
    #self._as.accept_new_goal()
    
    # start executing the action
    process = subprocess.Popen(goal.command, shell=True, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
    r = rospy.Rate(10);
    ret_code = None
    while ret_code == None:
      if self._as.is_preempt_requested():
        rospy.logerror('%s: Preempted' % self._action_name)
        process.kill() 
        self._result.output += process.stderr.read()  
        self._result.output += process.stdout.read()  
        self._as.set_preempted()
        break

      self._feedback.stderr = process.stderr.read()  
      self._feedback.stdout = process.stdout.read()  
      self._result.stderr += self._feedback.stderr 
      self._result.stdout += self._feedback.stdout 
      self._as.publish_feedback(self._feedback)
      ret_code = process.poll()
      r.sleep()

    if (ret_code == 0):
      self._result.exit_code=0
      rospy.loginfo('%s: Succeeded' % self._action_name)
      self._as.set_succeeded(self._result)
    else:
      print "Command Failed, Exit Code: ", ret_code
      self._result.exit_code = ret_code
      rospy.loginfo('%s: Failed' % self._action_name)
      self._as.set_aborted(self._result)

      
if __name__ == '__main__':
  rospy.init_node('shell_execution_action_server')
  ShellCommand(rospy.get_name())
  rospy.spin()

